#!/bin/sh
## ----------------------------------------------------------------------------
##                                                                           --
##                      GNADE  : GNu Ada Database Environment                --
##                                                                           --
##  Filename        : $Source: /cvsroot/gnade/gnade/bin/staticlib.sh,v $
##  Description     : build and install shared libs in the system            --
##  Author          : Michael Erdmann <Michael.Erdmann@snafu.de>
##  Created On      : 9-April-2002
##  Last Modified By: $Author: merdmann $
##  Last Modified On: $Date: 2002/04/14 11:31:21 $
##  Status          : $State: Exp $
##
##  Copyright (C) 2000-2002                                                  --
##                                                                           --
##  GNADE is copyrighted by the persons and institutions enumerated in the   --
##  AUTHORS file. This file is located in the root directory of the          --
##  GNADE distribution.                                                      --
##                                                                           --
##  GNADE is free software;  you can redistribute it  and/or modify it under --
##  terms of the  GNU General Public License as published  by the Free Soft- --
##  ware  Foundation;  either version 2,  or (at your option) any later ver- --
##  sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
##  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
##  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
##  for  more details.  You should have  received  a copy of the GNU General --
##  Public License  distributed with GNAT;  see file COPYING.  If not, write --
##  to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
##  MA 02111-1307, USA.                                                      --
##                                                                           --
##  As a special exception,  if other files  instantiate  generics from      --
##  GNADE Ada units, or you link GNADE Ada units or libraries with other     --
##  files  to produce an executable, these  units or libraries do not by     --
##  itself cause the resulting  executable  to  be covered  by the  GNU      --
##  General  Public  License.  This exception does not however invalidate    --
##  any other reasons why  the executable file  might be covered by the      --
##  GNU Public License.                                                      --
##                                                                           --
##  GNADE is implemented to work with GNAT, the GNU Ada compiler.            --
##                                                                           --
## ----------------------------------------------------------------------------

. ../../VERSION

SO_MAJOR=$GNADE_MAJOR
SO_MINOR=$GNADE_MINOR

gnadeinclude=../../lib-linux-gnu-i686/ada/adainclude 
targetdir="./"
packages=gnu-db

opt_clean="n"
opt_units="n"
opt_verbose=""
rootpkg="GNU.DB"
libname=""
##
##
##
usage()
{
	cat <<EOF
Usage: staticlib.sh [OPTIONS] <name>
Options:
	[--target=<targetdir> | -t <targetdir> ]
	[--packages=<pkgfile> | -p <pkg.files> ]
	[--clean] 

Description:
This script creates a static library including all modules
as specified by the -p switch. All modules with the name 
GNU.DB.<pkg> are included in building the named library.

The library will receive the name lib<name>.so.<major>.

EOF
	exit $1
}
##
## Create the module list
##
modulelist() {
   base=$1

   rm -rf modules.lst   
   for f in $base*.ads ; do
      g=`basename $f .ads` ;
      p=`grep -i "package $rootpkg.* is" ${f} | head -1 | awk '{print $2;}'` ;
      if [ "$p" != "" ]; then echo "$p" >> modules.lst ; fi ;
   done
	
   rm -f makelib.adb
   for f in `cat modules.lst | grep -i $rootpkg` ; do
      if [ "$f" != "" ]; then echo "with $f;" >> makelib.adb ; fi ;
   done ;
   echo "" >> makelib.adb
   echo "procedure MakeLib is begin null; end MakeLib;" >> makelib.adb
}

##
## Create a static library
##
makestatic() {
   libname=$1
   target=$libname.a

   mkdir obj_static
   (cd obj_static ; gnatmake -c -I.. -I$gnadeinclude makelib ) 2>log
   if test $? -gt 0 ; then
      echo "***** Compiler error ******"
      tail log
      ######## ERROR #######
      exit 1
      ######################
   fi

   ar cr $target obj_static/*.o
   ranlib $target
   chmod 0444 $target
}


##
## Copy a single unit 
##
copyunit() {
   t=$2/`basename $1` 
   if test -e $t -a ! -w $t ; then
      chmod 0666 $t
   fi;
   cp $opt_verbose $1 $t
   chmod 0444 $t
}

##
## Install into a target directory
##
installstatic() {
   libname=$1
   target=$targetdir
   lib=$libname.a

   mv $lib $target
 
   if test "$opt_units" = "y"; then
      for i in $packages*.ad[bs]; do
         copyunit $i $gnadeinclude 
      done

      for i in obj_static/*.ali; do
         copyunit $i $target
      done
      rm -rf obj_static
   fi
}

##
## Cleanup the local directory
##
cleanup(){
   rm -rf obj_static
   rm -rf $targetdir/lib$1.a
   rm -rf makelib.adb modules.lst
}

## ########################################## ##
##            Main Entry                      ##
## ########################################## ##

if test $# -eq 0; then
   usage 1 1>&2
   exit 1;
fi
##
## Scan the arguments
##
for i in $*
do
   case "$1" in
    -*=*) optarg=`echo "$1" | sed 's/[-_a-zA-Z0-9]*=//'` ;;
       *) optarg= ;;
   esac

   case $1 in
       --target=* )
          targetdir=$optarg
          ;;
       -t )
          shift
          targetdir=$1
	  ;;

       --packages=* )
          packages=$optarg
	  ;;

       -p ) 
          shift
	  packages=$1
	  ;;

       --include=* )
          gnadeinclude=$optarg
	  ;;

       --units)
          opt_units="y"
          ;;

       --verbose|-v)
          opt_verbose="-v"
          ;;

       --clean | -c )
          opt_clean="y"
          ;;

       --root=*) 
          rootpkg=$optarg
          ;;

       *) if test "$opt_clean" = "n"; then
             cleanup $1
             modulelist    $packages
             makestatic    lib$1
             installstatic lib$1
          else 
	     cleanup $1
	  fi
          ;;
   esac
   shift;
done

##
exit 0
## ################## ##
## #    E X I T     # ##
## ################## ##
