------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with 
  OCI.Thick.Statements;
--with  OCI.Thick.Connection_Pkg;
package OCI.Thick.Parameters is
use Statements;

   type Typecode is (
      TYPE_VARCHAR,
      TYPE_NUMBER,
      TYPE_INTEGER,
      TYPE_FLOAT,
      TYPE_DECIMAL,
      TYPE_VARCHAR2,
      TYPE_DATE,
      TYPE_REAL,
      TYPE_DOUBLE,
      TYPE_UNSIGNED8,
      TYPE_UNSIGNED16,
      TYPE_UNSIGNED32,
      TYPE_SIGNED8,
      TYPE_SIGNED16,
      TYPE_SIGNED32,
      TYPE_PTR,
      TYPE_OPAQUE,
      TYPE_RAW,
      TYPE_CHAR,
      TYPE_MLSLABEL,
      TYPE_OBJECT,
      TYPE_REF,
      TYPE_CLOB,
      TYPE_BLOB,
      TYPE_BFILE,
      TYPE_CFILE,
      TYPE_NAMEDCOLLECTION,
      TYPE_OCTET,
      TYPE_SMALLINT,
      TYPE_VARRAY,
      TYPE_TABLE);

-- Parameter

   type Parameter is new Handle_Reference with private;
   
   type Parameter_Array is array (Positive range<>) of Parameter;

   function Column (Stmt : Statement; Position : Positive) return Parameter;
   function Get_Parameters (Stmt : Statement) return Parameter_Array;

   function Name (Param : Parameter) return String;
   function Type_Code (Param : Parameter) return Typecode;
   function Type_Name (Param : Parameter) return String;
   function Data_Size (Param : Parameter) return Natural;
   function PRECISION (Param : Parameter) return Integer;
   function SCALE (Param : Parameter) return Integer;
   function IS_NULL (Param : Parameter) return Boolean;
   function SCHEMA_NAME (Param : Parameter) return String;

private
--   use Connection_Pkg;
   for typecode use (
      TYPE_FLOAT      => OCI_TYPECODE_FLOAT,
      TYPE_DATE       => OCI_TYPECODE_DATE,
      TYPE_REAL       => OCI_TYPECODE_REAL,
      TYPE_DOUBLE     => OCI_TYPECODE_DOUBLE,
      TYPE_SIGNED8    => OCI_TYPECODE_SIGNED8,
      TYPE_SIGNED16   => OCI_TYPECODE_SIGNED16,
      TYPE_SIGNED32   => OCI_TYPECODE_SIGNED32,
      TYPE_NUMBER     => OCI_TYPECODE_NUMBER,
      TYPE_DECIMAL    => OCI_TYPECODE_DECIMAL,
      TYPE_UNSIGNED8  => OCI_TYPECODE_UNSIGNED8,
      TYPE_UNSIGNED16 => OCI_TYPECODE_UNSIGNED16,
      TYPE_UNSIGNED32 => OCI_TYPECODE_UNSIGNED32,
      TYPE_OCTET      => OCI_TYPECODE_OCTET,
      TYPE_SMALLINT   => OCI_TYPECODE_SMALLINT,
      TYPE_INTEGER    => OCI_TYPECODE_INTEGER,
      TYPE_RAW        => OCI_TYPECODE_RAW,
      TYPE_CHAR       => OCI_TYPECODE_CHAR,
      TYPE_PTR        => OCI_TYPECODE_PTR,
      TYPE_VARCHAR2   => OCI_TYPECODE_VARCHAR2,
      TYPE_VARCHAR    => OCI_TYPECODE_VARCHAR,
      TYPE_MLSLABEL   => OCI_TYPECODE_MLSLABEL,
      TYPE_VARRAY      => OCI_TYPECODE_VARRAY,
      TYPE_TABLE        => OCI_TYPECODE_TABLE,
      TYPE_OBJECT        => OCI_TYPECODE_OBJECT,
      TYPE_OPAQUE         => OCI_TYPECODE_OPAQUE,
      TYPE_NAMEDCOLLECTION => OCI_TYPECODE_NAMEDCOLLECTION,
      TYPE_BLOB           => OCI_TYPECODE_BLOB,
      TYPE_BFILE         => OCI_TYPECODE_BFILE,
      TYPE_CLOB         => OCI_TYPECODE_CLOB,
      TYPE_REF        => OCI_TYPECODE_REF,
      TYPE_CFILE       => OCI_TYPECODE_CFILE);
      
   for typecode'Size use Integer'Size;
   
   type Parameter is new Handle_Reference with record
     Last_Stmt : Statement;
--     Connect : Connection;
   end record;
   
   procedure Destroy(Object : in out Parameter);

end OCI.Thick.PArameters;