------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with 
   OCI.Lib,
   OCI.Thread,
   OCI.Thick.Parameters;

package body OCI.Thick.string_var is  

   use lib,Parameters;
   use type OCIHandle;

   procedure Resize(Var : in out Variable; Length : Natural) is
      Value : aliased OCIString := OCIString(Get_Value_Internal(Var));
      rc : sword; 
      use type ub4;
   begin
      rc := OCIStringResize(
         env => thread.Environment,
         err => thread.Error,
         new_size => ub4(Length),
         str => Value'Access); 
      Check_Error(rc);
      Set_Value(Var,OCIHandle(Value));
   end Resize;   

   procedure Allocator (Value : in out Var_Type'Class; Size : in Natural) is
   begin
      Resize (Variable(Value), Size);
   end;

  procedure Initialize(Object : in out Variable) is
  begin
     Initialize(Var_Type(Object));
     Set_Value(Object,Empty_Handle);
  end;

  procedure Finalize  (Object : in out Variable) is
  begin
     if Get_Value_Internal(Object) /= Empty_Handle then
        Resize(Object,0);
     end if; 
     Finalize(Var_Type(Object));
  end Finalize;

  function Allocated_Size(Value : in Variable) return Natural is
     alsize : aliased ub4 := 0;
     Str : OCIString := OCIString(Get_Value_Internal(Value));
     rc : sword := OCIStringAllocSize(
                              env => Thread.Environment,
                              vs => Str,
                              allocsize => alsize'Access); 
                               
  begin
     Check_Error(rc);
     return Natural(alsize);
  end Allocated_Size;
  
  procedure Correct_Size(Stmt : Statement; Value : in out Variable; Position : Positive) is
    Param : Parameter := Column(Stmt,Position);
    NSize : Natural;
    TCode : Typecode;
  begin
     tcode := Type_Code(Param);
     if TCode=TYPE_VARCHAR2
     or TCode=TYPE_CHAR
     or TCode=TYPE_VARCHAR then
       NSize := Data_Size(Param)+1;
--       put_line(integer'image(nsize));
     else
       NSize := 64;  -- ot fanaria
     end if;  
     if Get_Value_Internal(Value) = Empty_Handle 
     or else Allocated_Size(Value)<nsize then 
       Resize(Value,NSize);
--       put_line("Allocated "&Integer'Image(Allocated_Size(Value)));
     end if;
  end Correct_Size;     

  procedure Define(Stmt : Statement; Value : in out Variable; Position : Positive) is
  begin
     if Described(Stmt) then
       Correct_Size(Stmt,Value,Position);
     end if;
     Define(Stmt,OCIHandle_Var.Var_Type(Value),Position);
  end Define;
  
  procedure Set_Value(Var : in out Variable; Value : String) is
    str : aliased OCIString := OCIString(Get_Value_Internal(Var));
    rc : sword := OCIStringAssignText (
         env => thread.Environment,
         err => thread.Error,
         rhs => C.To_C(Value), 
         rhs_len => Value'Length,
         lhs => str'Access);
  begin
    Check_Error(rc);
    Set_Value(Var,OCIHandle(str)); 
  end;
  
  function Get_Value(Var : Variable) return String is
    Env : OCIEnv := Thread.Environment;
    Str : OCIString := OCIString(OCIHandle_Var.Get_Value(Var));
  begin
     return C.To_Ada(CSTR.Value(
            OCIStringPtr(Env,Str), 
            C.Size_t(OCIStringSize(Env,Str))),
            False);
  end;

end OCI.Thick.string_var;
